function love.load()
	paths = {}
	images = {}
	colors = {}
	local t = love.filesystem.getDirectoryItems("images")
	for i, v in ipairs(t) do
		if string.sub(v, -4, -1) == ".jpg" or string.sub(v, -4, -1) == ".png" then
			table.insert(paths, "images/" .. v)
		end
	end
	
	math.randomseed(os.time())
	tableString = ""
	
	for i, v in ipairs(paths) do
		if i > 1 then tableString = tableString .. "\r\n" end
		tableString = tableString .. v .. " = "
		local img = love.image.newImageData(v)
		local r, g, b, a = getAverage(img)
		images[i] = love.graphics.newImage(v)
		table.insert(colors, {r, g, b, a})
		tableString = tableString .. "[" .. r .. ", " .. g .. ", " .. b .. ", " .. a .. "]"
	end
	
	selections = {}
	selected = false
	selMode = true --true = rgb, false = hsb
	
	love.keypressed("enter", {255, 255, 255})
end

function love.draw()
	if selected then
		love.graphics.setColor(255, 255, 255, 255)
		local off = 10
		local img = images[ selections[selected] ]
		local mw, mh = 800-2*off, 600-2*off
		local sw, sh = mw/img:getWidth(), mh/img:getHeight()
		local scale = math.min(sw, sh)
		
		love.graphics.draw(img, 400-img:getWidth()/2*scale, 300-img:getHeight()/2*scale, 0, scale, scale)
		
		local segs, dist = 16, 2
		love.graphics.setColor(0, 0, 0, 255)
		local px, py
		for i = 1, segs+1 do
			local offx = math.cos(i/segs*math.pi*2)*dist
			local offy = math.sin(i/segs*math.pi*2)*dist
			
			if i == segs+1 then
				love.graphics.setColor(255, 255, 255, 255)
				offx, offy = 0, 0
			end
			
			px = math.floor(400-img:getWidth()/2*scale)
			py = math.floor(300-img:getHeight()/2*scale)
			
			love.graphics.print((#selections > 1 and "[" .. selected .. "/" .. #selections .. "] " or "") .. (selected > 1 and "< " or "") .. paths[ selections[selected] ] .. ((#selections > 1 and selected < #selections) and " >" or ""), px+5+offx, py+5+offy)
			love.graphics.print("Compare mode [TAB]: " .. (selMode and "RGB" or "HSB"), px+5+offx, py+5+20+offy)
			
			px = math.floor(400+img:getWidth()/2*scale)-off-50
			py = math.floor(300+img:getHeight()/2*scale)-off-50
		
			love.graphics.print("Average", px+offx, py-20+offy)
		end
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.rectangle("fill", px-4, py-4, 58, 58)
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.rectangle("fill", px-2, py-2, 54, 54)
		love.graphics.setColor(colors[ selections[selected] ])
		love.graphics.rectangle("fill", px, py, 50, 50)
	end
	
	if love.keyboard.isDown("f1") then
		love.graphics.setColor(255, 255, 255, 105)
		love.graphics.rectangle("fill", 0, 0, 800, 600)
		love.graphics.setColor(0, 0, 0, 105)
		love.graphics.rectangle("fill", 0, 0, 800, 600)
		
		local list = {
			"Ctrl + V, space or enter to paste color from clipboard.",
			"Ctrl + R to select a random color.",
			"Tab to switch between RGB comparison and HSB comparison.",
			"Left or right to switch between images when multiple of them match.",
			"Ctrl + C to copy the list of average colors for each image."
		}
		
		local font = love.graphics.getFont()
		local ww = 0
		local hh = 20*#list
		local off = 10
		for i, v in ipairs(list) do
			ww = math.max(font:getWidth(v), ww)
		end
		
		local px, py = math.floor(400-ww/2), math.floor(300-hh/2)
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.rectangle("fill", px-off-2, py-off-2, ww+off*2+4, hh+off*2+4)
		love.graphics.setColor(155, 155, 155, 255)
		love.graphics.rectangle("fill", px-off, py-off, ww+off*2, hh+off*2)
		
		local segs, dist = 16, 2
		love.graphics.setColor(0, 0, 0, 255)
		for j = 1, segs+1 do
			local offx = math.cos(j/segs*math.pi*2)*dist
			local offy = math.sin(j/segs*math.pi*2)*dist
			
			if j == segs+1 then
				love.graphics.setColor(255, 255, 255, 255)
				offx, offy = 0, 0
			end
			
			for i, v in ipairs(list) do
				local px = math.floor(400-font:getWidth(v)/2)
				local py = math.floor(300-#list*20/2+10-font:getHeight()/2+(i-1)*20)
				
				love.graphics.print(v, px+offx, py+offy)
			end
		end
	end
end

function love.keypressed(key, ctable)
	if key == "tab" and selected then
		selMode = not selMode
		love.keypressed("enter", {love.graphics.getBackgroundColor()})
	elseif key == "c" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) then
		love.system.setClipboardText(tableString)
	elseif key == "r" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")) then
		love.keypressed("enter", {math.random(256)-1, math.random(256)-1, math.random(256)-1})
	elseif key == "d" or key == "right" then
		if selected then
			selected = selected + 1
			if selected > #selections then
				selected = #selections
			end
		end
	elseif key == "a" or key == "left" then
		if selected then
			selected = selected - 1
			if selected < 1 then
				selected = 1
			end
		end
	elseif key == "enter" or key == "return" or key == "kpenter" or key == "space" or (key == "v" and (love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl"))) then
		local c = love.system.getClipboardText()
		if (type(c) == "string" and string.len(c) >= 7) or type(ctable) == "table" then
			local t = string.split(string.gsub(c, " ", ""), ",")
			if type(ctable) == "table" then t = ctable end
			if #t == 4 or #t == 3 then
				local r, g, b, a = false, false, false, false
				if tonumber(t[1]) then
					r = tonumber(t[1])
					if tonumber(t[2]) then
						g = tonumber(t[2])
						if tonumber(t[3]) then
							b = tonumber(t[3])
							if t[4] then
								if tonumber(t[4]) then
									a = tonumber(t[4])
								else
									a = 255
								end
							else
								a = 255
							end
						end
					end
				end
				
				if r then
					if r%1==0 and g%1==0 and b%1==0 and a%1==0 then
						if r >= 0 and r <= 255 and g >= 0 and g <= 255 and b >= 0 and b <= 255 and a >= 0 and a <= 255 then
							print("Current color = [" .. r .. ", " .. g .. ", " .. b .. ", " .. a .. "]")
							local hue, sat, bri = hsb(r, g, b)
							--FINALLY, all checks out
							
							love.graphics.setBackgroundColor(r, g, b)
							
							local max = math.max(r, g, b, a)
							
							local dif = false
							local dif2 = false
							for i, v in ipairs(colors) do
								local cr, cg, cb, ca = unpack(v)
								
								local chue, csat, cbri = hsb(cr, cg, cb)
								
								local d = math.abs(r-cr) + math.abs(g-cg) + math.abs(b-cb) + math.abs(a-ca)
								if not dif then
									dif = {{i}, d}
								elseif d == dif[2] then
									table.insert(dif[1], i)
								elseif d < dif[2] then
									dif = {{i}, d}
								end
								
								
								local d2 = (math.abs(chue-hue))/(math.pi) + math.abs(csat-sat) + math.abs(cbri-bri)
								if not dif2 then
									dif2 = {{i}, d2}
								elseif d2== dif2[2] then
									table.insert(dif2[1], i)
								elseif d2 < dif2[2] then
									dif2 = {{i}, d2}
								end
							end
							
							if selMode then
								selections = dif[1]
							else
								selections = dif2[1]
							end
							selected = 1
						end
					end
				end
			end
		end
	end
end

function hsb(r, g, b) --I STOLED this from Pattern Generator. fite me
	local c = {r, g, b}
	local hue, sat, bri
	
	sat = math.min(unpack(c))/math.max(unpack(c))
	if math.max(unpack(c)) == 0 then
		sat = 0
	end
	bri = 1-math.max(unpack(c))/255
	
	local r, g, b = unpack(c)
	
	r, g, b = r-math.min(r, g, b), g-math.min(r, g, b), b-math.min(r, g, b)
	if math.max(r, g, b) > 0 then
		r, g, b = r/math.max(r, g, b), g/math.max(r, g, b), b/math.max(r, g, b)
	end
	
	if r <= 0 then r = 0 end
	if g <= 0 then g = 0 end
	if b <= 0 then b = 0 end
	
	--=====--
	-- HUE --
	--=====--
	local aa = math.pi/2
	hue = aa
	if r == 1 then
		hue = aa
		if g > 0 then
			hue = hue + g*(aa/3*2)
		else
			hue = hue - b*(aa/3*2)
		end
	elseif g == 1 then
		hue = math.pi+aa/3
		if b > 0 then
			hue = hue + b*(aa/3*2)
		else
			hue = hue - r*(aa/3*2)
		end
	elseif b == 1 then
		hue = math.pi*2-aa/3
		if r > 0 then
			hue = hue + r*(aa/3*2)
		else
			hue = hue - g*(aa/3*2)
		end
	end
	
	if hue < 0 then
		hue = hue + math.pi*2
	elseif hue >= math.pi*2 then
		hue = hue - math.pi*2
	end
	
	sat = 1-sat
	bri = 1-bri
	
	return hue, sat, bri
end

function getAverage(img)
	local rr, rg, rb, ra = 0, 0, 0, 0
	for x = 1, img:getWidth() do
		for y = 1, img:getHeight() do
			local r, g, b, a = img:getPixel(x-1, y-1)
			rr = rr+r; rg = rg+g; rb = rb+b; ra = ra+a
		end
	end
	local i = img:getWidth()*img:getHeight()
	rr = rr/i;rg = rg/i; rb = rb/i; ra = ra/i
	rr = math.floor(rr); rg = math.floor(rg); rb = math.floor(rb); ra = math.floor(ra)
	return math.max(0, math.min(255, rr)), math.max(0, math.min(255, rg)), math.max(0, math.min(255, rb)), math.max(0, math.min(255, ra))
end

function string.split(st, del)
	local del = del or ","
	local ret = {""}
	local wait = 0
	for i = 1, string.len(st) do
		local c = string.sub(st, i, i)
		if wait == 0 then
			if string.sub(st, i, i+string.len(del)-1) == del then
				table.insert(ret, "")
				wait = string.len(del)-1
			else
				ret[#ret] = ret[#ret] .. c
			end
		else
			wait = wait-1
		end
	end
	return ret
end